﻿function getListItemIdFromProfileId
{[CmdletBinding()] 
param (
[object] $SPList,
[parameter(mandatory=$true)]
[alias("id")] 
[int] $ProfileId
)		
	Write-Verbose "Looking for list item with id = $ProfileId"
	$count = ($SPList).Items.Count
	Write-Verbose "List count = $count"
	$i = 0;		
	while($i -lt $count)
	{			
		$curId = ($SPlist.Items[$i])["ProfileID"]
		write-Verbose "curid = $curId"
		if ( $curId -eq $ProfileId)
		{
			$rc = ($SPlist.Items[$i]).ID
			write-Verbose "List Item ID = $rc"
			return $rc
		}
		$i++
	}
	return $null;
}

function GetPassword 
{		
	$prompted = read-host -Prompt "Enter Password for SASTrust" -AsSecureString
	$rc = [Runtime.InteropServices.Marshal]::PtrToStringAuto([Runtime.InteropServices.Marshal]::SecureStringToBSTR($prompted))
}

function StartSharePointAdminService
{
	$rc = 0
	try
	{
		if ( ((Get-Service SPAdminV4).Status -ne "Running"))
		{
			Write-Verbose "SPAdminV4 service is not Started, starting now..."
			Start-Service SPAdminV4
			Start-Sleep 4
		
			if ( ((Get-Service SPAdminV4).Status -ne "Running"))
			{
				Throw "Unable to start SharePoint Admin service"
			}
		}
		Write-Verbose "SPAdmin service is running"
	}
	catch
	{
		$rc = -12
	}
	return $rc	
}
function encryptPW($pw)
{    
    [void][System.Reflection.Assembly]::Load("SASWebParts, Version=7.100.1.0, Culture=neutral, PublicKeyToken=be58efc3b934219b")
    $enc =  [SAS.WebParts.Utilities.ProfileManager]::EncryptPW($pw);   
    return $enc;
}

function WaitForSolutionDeployState {
[CmdletBinding()] param ([bool] $state = $true, [int] $times = 50)		

	try {	
	    write-Verbose "parms = $state - $times - $SASSolutionID"
		$sol = get-spsolution -id $SASSolutionID 2> $null
		if ($sol -eq $null )
		{ 
			Write-Verbose "Solution $SASSolutionID not found"
			return $false;
		}
	}
	catch  
	{
	   return $false;
	}
	$IsDeployed = $sol.deployed 
	$i = 0
	$interval = [int]$times/10
	while ( $i -lt $times -and ($IsDeployed -ne $state) )
	{
		Write-Verbose "Sleeping... $interval seconds"
		Start-Sleep $interval
		$i++
		$IsDeployed = $sol.deployed 		
	}

	Write-Verbose "End WaitForSolutionDeployState state = $IsDeployed"	
	$IsDeployed
}

function AddSharePointSnapIn
{
	$rc = 0
	$snap = Get-PSSnapIn -Name "Microsoft.SharePoint.PowerShell" 2> $null
	if ( $null -eq $snap )
	{ 
		Write-Verbose "Loading SharePoint SnapIn"
		Add-PSSnapIn -Name "Microsoft.SharePoint.PowerShell" 
		$snap = Get-PSSnapIn  -Name "Microsoft.SharePoint.PowerShell"
	}
	#if still null then snapin was not found return error code
	if ( $snap -eq $null )
	{ 
		Write-Host "Getting all registered snapins"
	    Get-PSSnapin -registered
		$rc = -1
		Start-Sleep -Seconds 10
	}
	$rc
}
function GetNextProfileId 
{[CmdletBinding()]
param 
(
[Parameter(Mandatory=$True)]
[string]$SiteUrl)
	$rc = 0
	$gcs = Start-SPAssignment
	try
	{	
		$SPSite = $gcs | Get-SPWeb $SiteUrl
		$listCount = $SPSite.Lists.Count	
		$SPList = $SPSite.Lists[$SASListInstanceID]
		if ($SPList -ne $null)
		{
			$count = ($SPList).Items.Count
			Write-Host "List count = " $count
			$i = 0;		
			while($i -lt $count)
			{			
				$curId = ($SPlist.Items[$i])["ProfileID"]
				Write-Verbose "Profile ID =  $curId  is used"
				if ( $curId -eq $rc )
				{
				   $rc++
				   $i = 0
				}
				else
				{
					$i++
				}
			}
		}
		else
		{	Throw "Unable to find list $SASListInstanceID" }
		write-Verbose "Available ID =  $rc"
	}
	catch
	{
		write-host "error occured: $_"
		$rc = -2
	}
	finally { 
		write-verbose "Stopping assignment"
		Stop-SPAssignment  $gcs 
	}
	return [int]$rc
}
###################################### public methods  #####################################################
function Add-SASDefaultProfile
{
<#
.SYNOPSIS
Creates the default SAS Metadata profile and adds it to the web parts profile list
.DESCRIPTION
Creates the default SAS Metadata profile and adds it to the profile list. If the item already exits an error will result. Use Remove-SASProfile to remove the current default first
.EXAMPLE
Add-SASDefaultProfile -SiteUrl http://mysite.com  -ProfileName "Default Profile" -ServiceUrl "https://myinternalsite.com/SASWebPartsService/Service.svc" -ServerName "sasserver" -SASTrust "SASTrust@saspw" 
.PARAMETER SiteUrl
The url for the SharePoint site collection that hosts the list
.PARAMETER ProfileName
(Optional) The name to use for the initial default profile. Default is "Default Site Profile"
.PARAMETER ServiceUrl
The url of the SAS WebParts Service 
.PARAMETER ServerName
The network name of the SAS Metadata server 
.PARAMETER Port
(Optional) The Port number of the SAS Metadata server. The default is 8561
.PARAMETER SASTrust
The ID of the SAS Trusted User for the SAS Metadata server
.PARAMETER Password
(Optional) The Password for the Trusted User. If not specified will be prompted for. Can be specified in sas002 format. This will be encrypted in the profile list
.PARAMETER ImpersonationID
(Optional) If specified this ID will be used by all users when connecting to metadata. The default is for each user to connect with their SharePoint ID
#>
[CmdletBinding()] 
param (
[Parameter(Mandatory=$True)]
[Alias('url')]
[string] $SiteUrl,
[Alias('pn')] 
[string] $ProfileName, 
[Parameter(Mandatory=$True)]
[Alias('svc')]
[string] $ServiceUrl, 
[Parameter(Mandatory=$True)]
[Alias('server')]
[string] $Servername, 
[Alias("P")]
[int]    $Port = 8561,
[Parameter(Mandatory=$True)]
[Alias("id")]
[string] $SASTrust, 
[Alias("pw")]
[string] $Password, 
[Alias("ImpID")]    
[string] $impersonationID
)
	return Add-SASProfile -SiteUrl $SiteUrl -ProfileId -1 -ProfileName $ProfileName -ServiceUrl $ServiceUrl -Servername $Servername -Port $Port -SASTrust $SASTrust -Password $Password -ImpersonationId $ImpersonationId 
}

function Add-SASProfile
{
<#
.SYNOPSIS
Adds a SAS Metadata profile to the webparts profile list
.DESCRIPTION
Creates a SAS Metadata profile and adds it to the profile list
.EXAMPLE
Add-SASProfile -SiteUrl http://mysite.com  -ProfileName "Test" -ServiceUrl "https://myinternalsite.com/SASWebPartsService/Service.svc" -ServerName "sasserver" -SASTrust "SASTrust@saspw" 
.PARAMETER SiteUrl
The url for the SharePoint site collection that hosts the list
.PARAMETER ProfileId
(Optional) The Id to use. If not specified the next available will be used. The Id must be unique.
.PARAMETER ProfileName
(Optional) The name to use for the initial default profile. Default is "Profile ID" where ID is the ProfileId
.PARAMETER ServiceUrl
The url of the SAS WebParts Service 
.PARAMETER ServerName
The network name of the SAS Metadata server 
.PARAMETER Port
(Optional) The Port number of the SAS Metadata server. The default is 8561
.PARAMETER SASTrust
The ID of the SAS Trusted User for the SAS Metadata server
.PARAMETER Password
(Optional) The Password for the Trusted User. If not specified will be prompted for. Can be specified in sas002 format. This will be encrypted in the profile list
.PARAMETER ImpersonationID
(Optional) If specified this ID will be used by all users when connecting to metadata. The default is for each user to connect with SharePoint ID
#>
[CmdletBinding()] 
param (
[Parameter(Mandatory=$True)]
[Alias('url')]
[string] $SiteUrl, 
[Alias('pid')]
[int]    $ProfileId = -2, 
[Alias('pn')] 
[string] $ProfileName, 
[Parameter(Mandatory=$True)]
[Alias('svc')]
[string] $ServiceUrl, 
[Parameter(Mandatory=$True)]
[Alias('server')]
[string] $Servername,
[Alias("P")] 
[int]    $Port = 8561,
[Parameter(Mandatory=$True)]
[Alias("id")]
[string] $SASTrust, 
[Alias("pw")]
[string] $Password, 
[Alias("ImpID")] 
[string] $impersonationID
)
	if($ProfileId -lt -1)
	{   
		Write-Verbose "Getting next available ID"
		$ProfileId = GetNextProfileId $SiteUrl 
	}
	
    #verfiy profile id
	Write-Verbose "Adding Profile id $ProfileId"
	
	# Get Site
	$gcs = Start-SPAssignment
	try
	{	
		$SPSite = $gcs | Get-SPWeb $SiteUrl
		if ($SPSite -eq $null)
		{   return -11 }
	   
		# Get List
		Write-Verbose "Retrieving list $SASListInstanceID"
		$SPList = $SPSite.Lists[$SASListInstanceID]
		if ($SPList -ne $null)
		{
			if ( $Password -eq $null -or $Password.length -eq 0)
			{
				Write-verbose "Prompt for password"
				$Password = getPassword
			}
			else
			{ Write-Verbose "Password Provided" }
				
			try{	
				$encrypt = encryptPW($Password)		
			}
			catch
			{
				write-host $_
				return  -9			
			}
			if ( $ProfileName.length -eq 0 )
			{
				$ProfileName = "Profile " + $ProfileId
			}
			try
			{
				$SPListItem = $SPList.Items.Add()
				$SPListItem["ProfileID"] = $ProfileId
				$SPListItem["Title"] = $ProfileName
				$SPListItem["ProfileName"] = $ProfileName
				$SPListItem["ServiceUrl"] = $ServiceUrl
				$SPListItem["Server"] = $ServerName
				$SPListItem["ServerPort"] = $Port
				$SPListItem["SASTrustUser"] = $SASTrust
				$SPListItem["SASTrustPW"] = $encrypt        
				$SPListItem["ImpersonationID"] = $impersonationID				
				$SPListItem.Update()
			}
			catch
			{ 
				write-host $_ 
				return -10; 
			}
		}
		else
		{
			return -15
		}
	}
	finally
	{
		Stop-SPAssignment  $gcs 		
	}
	return 0
}

function Remove-SASProfile
{
<#
.SYNOPSIS
Deletes the specified ID from the SAS web parts Profile list
.DESCRIPTION
Deletes the specified ID from the SAS web parts Profile list
.EXAMPLE
Remove_SASProfile -pid 1 -SiteUrl http://mysite.com
.PARAMETER ProfileId
The Profile Id to delete. 
.PARAMETER SiteUrl
The url for the SharePoint site collection that hosts the list
#>
[CmdletBinding(
	ConfirmImpact='High',
	SupportsShouldProcess=$true)] 
	param (
		[parameter(mandatory=$true)]
		[alias("pid")]
		[int]$ProfileId,		
		[parameter(mandatory=$true)]
		[alias("url")]
		[string]$SiteUrl)
	
	# Get Site
	$gcs = Start-SPAssignment
	try
	{	
		$SPSite = $gcs | Get-SPWeb $SiteUrl
		if ($SPSite -eq $null)
		{   return -11 }

		$SPList = $SPSite.Lists[$SASListInstanceID]	
		if ( $SPList -eq $Null )
		{
			Write-Host "The SAS web parts profile list does not exist on site $SiteUrl"
			return -15
		}
		if ($SPList.Items.Count -gt 0 )
		{	    
			$listndx = getListItemIdFromProfileId $SPList -id $ProfileId
			if ($listndx)
			{
				$SPListItem = $SPList.Items.GetItemById( $listndx)
				$name = $SPListItem["ProfileName"]
				if ( $PSCmdlet.ShouldProcess("Profile Id: $ProfileId - Profile Name: $name","Delete Profile"))
				{
					$SPlist.Items.DeleteItemById($listndx)
					$SPlist.Update()
				}
			}
			else
			{ 
				write-host "The Profile with id $ProfileId does not exist in the list"
				return -14 
			}
		}
	}
	finally
	{
		Stop-SPAssignment  $gcs 		
	}
	return 0
}

function Get-SASProfiles
{
<#
.SYNOPSIS
Lists the SAS profiles for the specified site collection
.DESCRIPTION
Lists the SAS Metadata profiles that are available to the SAS web parts for the specified site
.EXAMPLE
Get-SASProfiles http://mysite.com
.PARAMETER SiteUrl
The url for the SharePoint site collection that hosts the list
#>
[CmdletBinding()] 
	param (
		[parameter(mandatory=$true)]
		[alias("url")]
		[string]$SiteUrl
	)
	$exitCode = 0
	$gcs = Start-SPAssignment
	try
	{
		$SPSite = $gcs | Get-SPWeb $SiteUrl 2> $null
		if ($SPSite -eq $null)
		{
			Write-Verbose "SiteUrl $SiteUrl not found"
			$exitCode = -11
			Throw "SiteUrl $SiteUrl not found";				
		}
		$spl = $SPSite.Lists["$SASListInstanceID"]
		if ( $spl -eq $null )
		{
			Write-Verbose "List $SASListInstanceID not found"
			$exitCode = -15
			Throw "List $SASListInstanceID not found";
		}
		$count = ($spl.Items).Count
		write-Verbose "number of SASProfiles = $count"
		if ( $count -gt 0 )
		{
			$list = @()
			$items = $spl.Items
			$items | foreach-object {
				$ht = @{}
				$ht
				$ht.ID = $_["ProfileID"]
				$ht.ProfileName = $_["ProfileName"]
				$ht.ServiceUrl = $_["ServiceUrl"]
				$ht.Server = $_["Server"]			
				$ht.Port = $_["ServerPort"]
				$ht.SASTrustId = $_["SASTrustUser"]
				$ht.impersonationID = $_["ImpersonationID"]				
				$list += $ht			
			}
			return $list
		}
		else
		{
			Write-Host "There are no profiles in the list"
		}
	}
	finally
	{
		Stop-SPAssignment  $gcs 		
	}
}
function Remove-SASProfileList
{
<#
.SYNOPSIS
Deletes the SAS profile list from the specified site
.DESCRIPTION
Deletes the SAS profile list from the specified site
.EXAMPLE
Remove-SASProfileList -url 'http://spsitecol.com' 
.PARAMETER SiteUrl
The url for the SharePoint site to remove the list from
#>
[CmdletBinding()]
param 
(
[Parameter(Mandatory=$True)]
[Alias('url')]
[string]$SiteUrl
)
	$exitCode = 0
	$gcs = Start-SPAssignment
	try
	{
		$SPSite = $gcs | Get-SPWeb $SiteUrl
		if ($SPSite -eq $null)
		{
			write-error "The web $SiteUrl was not found";   
			return -11;
		}
		
		#delete the list
		Write-Verbose "Delete MDProfile List: id = $SASListInstanceID"
		$SPList = $SPSite.Lists[$SASListInstanceID]
		if ($SPList -ne $null)
		{
		   $SPSite.Lists.Delete($SPList.ID)
		   Write-Verbose "List deleted"
		}		
		else
		{ Write-Warning "List does not exist at this site" }
	}
	finally
	{
		Stop-SPAssignment  $gcs 		
	}
	return $exitCode
}

function Uninstall-SASWebParts
{
<#
.SYNOPSIS
Uninstalls the SAS WebParts version 6.13
.DESCRIPTION
Uninstalls the SAS Content Viewer Webpart (version 6.13) and SAS Central Webpart (version 6.13) 
.EXAMPLE
Unnstall-SASWebParts -url 'http://spsitecol.com' 
.PARAMETER SiteUrl
The url for the SharePoint site to uninstall the web parts
.PARAMETER WebApp
(Optional) The SharePoint Web Application to uninstall the solution. The default is -AllWebApplications
#>
[CmdletBinding()]
param 
(
[Parameter(Mandatory=$True)]
[Alias('url')]
[string]$SiteUrl,
[Alias("app")]
[string]$WebApp
)
	$exitCode = 0	
	try
	{
		Write-Verbose "Uninstall-SASWebParts - Input Parameters"
		Write-Verbose "SiteUrl = $SiteUrl"
		if ( $WebApp )	
		{Write-Verbose "WebApp = $WebApp"}
		else
		{Write-Verbose "WebApp = -AllWebApplications"}
		
		#start the SharePoint AdminV4 service if not running
		$exitCode = StartSharePointAdminService
		if ( $exitCode -ne 0 )
		{ 	Throw "Unable to start SharePoint AdminV4 service. Verify the SharePoint installation on this computer, and start the service" }

		#load sharepoint snapin if not installed 
		$exitCode = AddSharePointSnapIn
		if ( $exitCode -ne 0 )
		{ 	Throw "Unable to load SharePoint SnapIn. Verify SharePoint is installed on this computer" }
		
		#make sure it is  installed
		if ((Get-SPSolution $SASSolutionID 2> $null) -eq $null )
		{	$exitCode = -16; Throw "SAS WebParts Solution $SASSolutionID is not installed"; }
				
		Write-Host "Uninstalling SAS WebParts from Site $SiteUrl"
		Write-Verbose "Disable SASCentral webpart"
		Disable-SPFeature -Identity $SASCentralFeatureID -Url $SiteUrl -Confirm:$false
		Uninstall-SPFeature -Identity $SASCentralFeatureID  -force -Confirm:$false

		Write-Verbose "Disable SASContentWebPart"
		Disable-SPFeature -Identity $SASContentViewerFeatureID -Url $SiteUrl -Confirm:$false
		Uninstall-SPFeature -Identity $SASContentViewerFeatureID -force -Confirm:$false

		#S1083179: SAS Content Type is not uninstalled from sp 2013 sharepoint server after uninstall 
		# We must remove the list instance before removing the list feature, otherwise the content type does not get 
		# removed since it is in use when the feature is removed
		Write-Verbose "Removing the SAS Profile List instance";
		$exitCode = Remove-SASProfileList $SiteUrl
		if ( $exitCode -ne 0 )
		{
			Write-Warning "Return code from Remove-SASProfileList was $exitCode - the list may not have been deleted";
		}
		Write-Verbose "Disable the list feature"
		Disable-SPFeature -Identity $SASListFeatureID -Url $SiteUrl -Confirm:$false
		Uninstall-SPFeature -Identity $SASListFeatureID -force -Confirm:$false
		
		Write-Verbose "uninstall the solution"
		if ( -not $WebApp )
		{
			Uninstall-SPSolution -Identity $SASSolutionID -AllWebApplications -Confirm:$false
		}
		else
		{
			Uninstall-SPSolution -Identity $SASSolutionID -WebApplication $WebApp -Confirm:$false
		}
		$deployed = WaitForSolutionDeployState $False 120
		Write-Verbose "Solution Deployed = $deployed"		
		if ( $deployed -eq $True )
		{ $exitCode = Throw "Uninstall of Solution failed or is still pending" }
		
		Write-Verbose "remove spsolution"
		Remove-SPSolution -Identity $SASSolutionID -force -Confirm:$false
		$gcs = Start-SPAssignment
		try
		{
			$SPSite = $gcs | Get-SPWeb $SiteUrl
			if ($SPSite -eq $null)
			{   return -11 }
					
			
			Write-Verbose "Deleting SAS web parts from the web part gallery";
			$deleted = $false;
			$centralwp = $SPSite.GetFile("_catalogs/wp/SASCentralWebPart.webpart");
			if ( $centralwp -ne $null )
			{
				$centralwp.Delete();
				$deleted = $true;
				Write-Verbose "SAS Central web part deleted"
			}
			$contentwp = $SPSite.GetFile("_catalogs/wp/SASContentWebPart.webpart");
			if ( $contentwp -ne $null )
			{
				$contentwp.Delete();
				$deleted = $true;
				Write-Verbose "SAS Content Viewer web part deleted"
			}
			
			if ( $deleted -eq $false )
			{ write-Verbose "There were no SAS web parts found in the Web Part Gallery"}
			$exitCode = 0;
		}
		finally
		{
			Stop-SPAssignment  $gcs 		
		}	
	}
	finally
	{
		Write-Verbose "Uninstall of SAS WebParts is complete"
	}
	$exitCode
}

function Enable-SASWebParts
{
<#
.SYNOPSIS
Activates the SAS web part features on a site collection.
.DESCRIPTION
Activates the SAS web part features on a site collection. The webparts must first be installed on the Server. 
.EXAMPLE
Deploy-SASWebParts -url 'http://spsitecol.com' -svc 'https://serviceaddress.com/SASWebPartsService/Service.svc' -server 'sasmdserver' -p 8561 -id 'sastrust@saspw' 
.PARAMETER SiteUrl
The url for the SharePoint site to install the webparts
.PARAMETER ServiceUrl
The url of the SAS WebParts Service 
.PARAMETER ServerName
The network name of the SAS Metadata server 
.PARAMETER Port
(Optional) The Port number of the SAS Metadata server. The default is 8561
.PARAMETER SASTrust
The ID of the SAS Trusted User for the SAS Metadata server
.PARAMETER Password
The Password for the Trusted User. If not specified will be prompted for. Can be specified in sas002 format. This will be encrypted in the profile list
.PARAMETER ProfileName
(Optional) The name to use for the initial default profile. Default is "Default Site Profile"
.PARAMETER PackageFile
(Optional) The path to the SASWebParts.wsp. Default is '.\SASWebParts.wsp'
.PARAMETER ImpersonationID
(Optional) If specified this ID will be used by all users when connecting to metadata. The default is for each user to connect with SharePoint ID
.PARAMETER Upgrade
(Optional) If specified the current default profile, if it exists, will not be modified
#>
[CmdletBinding()]
param 
(
[Parameter(Mandatory=$True)]
[Alias('url')]
[string]$SiteUrl,
[Parameter(Mandatory=$True)]
[Alias("svc")]
[string]$ServiceUrl,
[Parameter(Mandatory=$True)]
[Alias("server")]
[string]$ServerName,    
[Alias("P")]
[int]$Port = 8561,
[Parameter(Mandatory=$True)]
[Alias("id")]
[string]$SASTrust,
[Alias("pw")]
[string]$Password,
[Alias("pn")]
[string]$ProfileName = "Default Site Profile",
[Alias("ImpID")]    
[string]$ImpersonationID,
[Alias("Up")]
[switch]$Upgrade
)
	Write-Verbose "Entering Enable-SASWebParts"	
	$exitCode = 0
	try
	{		
		#Site URL's seem to be case sensitive in some cases. If we get the spweb which isn't case sensitive and ask it for the
		#URL, that gives the correct casing
		write-host "Input site url = $SiteUrl";
		$gcsWeb = Start-SPAssignment
		try
		{	
			$web = get-spweb $SiteUrl;		
			if ( $web -eq $null)
			{
				$exitCode = -17
				throw "Unable to get Web at $SiteUrl"			
			}			
			$SiteUrl = $web.Url;
		}
		finally
		{
			Stop-SPAssignment  $gcsWeb 		
		}		
		#ok, what is the site url now?
		write-host "New SiteUrl = $SiteUrl";

		if ( $Upgrade )
		{
			Write-Host "Upgrading SAS Web Parts to site collection $SiteUrl"
		}
		else
		{
			Write-Host "Deploying SAS Web Parts to site collection $SiteUrl"
		}

		#make sure solution is already installed and deployed
		$SPSolution = Get-SPSolution $SASSolutionID 2> $null
		if ($SPSolution -eq $null )
		{	$exitCode = -16; Throw "SAS WebParts Solution $SASSolutionID is not installed on $SiteUrl"; }
		if ($SPSolution.Deployed -eq $False)
		{	$exitCode = -16; Throw "SAS WebParts Solution $SASSolutionID is not deployed on $SiteUrl"; }
		
		#deploy webparts Profile List
		if ((Get-SPFeature $SASListFeatureID 2> $null) -eq $null )
		{	$exitCode = -6; Throw "Feature SASWebPartsProfileList was not installed"; }
		
		try
		{
			$EnableList = $True
			#if in upgrade mode, this will already be enabled, don't enable
			if ($Upgrade)
			{
				#check status of list feature
				$EnableList = -not ( (Get-SPFeature -Identity $SASListFeatureID).status -eq "Online" )
			}
			
			if ( $EnableList )
			{
				Write-Host "Activating feature SASWebPartsProfileList on Site Collection $SiteUrl"
				Enable-SPFeature -Identity $SASListFeatureID -Url $SiteUrl -ErrorAction Stop -Force
				Write-Verbose "Feature SASMDProfile List Activated"
			}				
		}
		catch [System.Management.Automation.ActionPreferenceStopException]
		{
			$exitCode = -6;
			Write-Error "Unable to Enable feaure SASWebPartsProfileList";
			throw;	
		}			
		
		
		#deploy content viewer WebPart
		Write-Host "Activating feature SASContentViewer WebPart on Site Collection $SiteUrl"
		Enable-SPFeature -Identity $SASContentViewerFeatureID -Url $SiteUrl -Force
		if ((Get-SPFeature $SASContentViewerFeatureID 2> $null) -eq $null )
		{	$exitCode = -7; Throw "Unable to Enable feaure SAS Content Viewer WebPart"; }
		Write-Verbose "Feature SAS Content Viewer WebPart Activated"

		#deploy SASCentral WebPart
		Write-Host "Activating feature SASCentral WebPart on Site Collection $SiteUrl"
		Enable-SPFeature -Identity $SASCentralFeatureID -Url $SiteUrl -Force
		if ((Get-SPFeature $SASCentralFeatureID 2> $null) -eq $null )
		{	$exitCode = -8; Throw "Unable to Enable feaure SAS Central WebPart"; }
		Write-Verbose "Feature SAS Central WebPart Activated"
		
		$AddProfile = $True
		if ( $Upgrade )
		{
			#need to get the list object
			$gcs = Start-SPAssignment
			try
			{	
				$SPSite = $gcs | Get-SPWeb $SiteUrl
				if ($SPSite -eq $null)
				{   return -11 }

				$SPList = $SPSite.Lists[$SASListInstanceID]	
				if ( $SPList -eq $Null )
				{
					Write-Error "The SAS web parts profile list does not exist on site $SiteUrl"
					return -15
				}
				if ($SPList.Items.Count -gt 0 )
				{	    
					$listndx = getListItemIdFromProfileId $SPList -id '-1'
					if ($listndx)
					{
						#check for default sas profile if found, don't add
						write-host "The default profile already exists, don't add it"
						$AddProfile = $False
					}						
				}
			}
			finally
			{
				Stop-SPAssignment  $gcs 		
			}
		}
		
		if ( $AddProfile )
		{
			#now add the default site metadata profile      
			Write-Host "Adding the default site metadata profile"
			$exitcode = Add-SASDefaultProfile -SiteUrl $SiteUrl -ProfileName $ProfileName -ServiceUrl $ServiceUrl -Servername $ServerName -Port $Port -SASTrust $SASTrust -Password $Password -impersonationID $ImpersonationID
			if ( $exitCode -ne 0 )
			{ throw "Unable to Add Default Profile to List" }
			Write-Verbose "Profile Added to List"
		}
	}
	catch
	{
		Write-Error	$_
	}
	finally
	{
		Write-Verbose "Exiting Enable-SASWebParts"
	}
	return $exitCode
}

function Install-SASWebParts 
{
	<#
	.SYNOPSIS
	Installs the SAS WebParts version 6.13
	.DESCRIPTION
	Installs the SAS Content Viewer Webpart (version 6.13) and SAS Central Webpart (version 6.13) ontothe specified site collection and initiates the SAS Profile List with the default site profile
	.EXAMPLE
	Install-SASWebParts -url 'http://spsitecol.com' -svc 'https://serviceaddress.com/SASWebPartsService/Service.svc' -server 'sasmdserver' -p 8561 -id 'sastrust@saspw' -pw 'sas002[stpw]'
	.PARAMETER SiteUrl
	The url for the SharePoint site to install the webparts
	.PARAMETER WebApp
	(Optional) The SharePoint Web Application to install the solution. The default is -AllWebApplications
	.PARAMETER ServiceUrl
	The url of the SAS WebParts Service 
	.PARAMETER ServerName
	The network name of the SAS Metadata server 
	.PARAMETER Port
	(Optional) The Port number of the SAS Metadata server. The default is 8561
	.PARAMETER SASTrust
	The ID of the SAS Trusted User for the SAS Metadata server
	.PARAMETER Password
	The Password for the Trusted User. If not specified will be prompted for. Can be specified in sas002 format. This will be encrypted in the profile list
	.PARAMETER ProfileName
	(Optional) The name to use for the initial default profile. Default is "Default Site Profile"
	.PARAMETER PackageFile
	(Optional) The path to the SASWebParts.wsp. Default is '.\SASWebParts.wsp'
	.PARAMETER ImpersonationID
	(Optional) If specified this ID will be used by all users when connecting to metadata. The default is for each user to connect with SharePoint ID
	.PARAMETER Upgrade
	(Optional) If specified the script will attempt to uninstall the webparts before running the install
	#>
	[CmdletBinding()]
	param 
	(
	[Parameter(Mandatory=$True)]
	[Alias('url')]
	[string]$SiteUrl,
	[Alias("app")]
	[string]$WebApp,
	[Parameter(Mandatory=$True)]
	[Alias("svc")]
	[string]$ServiceUrl,
	[Parameter(Mandatory=$True)]
	[Alias("server")]
	[string]$ServerName,    
	[Alias("P")]
	[int]$Port = 8561,
	[Parameter(Mandatory=$True)]
	[Alias("id")]
	[string]$SASTrust,
	[Alias("pw")]
	[string]$Password,
	[Alias("pn")]
	[string]$ProfileName = "Default Site Profile",
	[string]$PackageFile =  ([string]$pwd) +  "\SASWebParts.wsp",
	[Alias("ImpID")]    
	[string]$ImpersonationID,
	[Alias("up")]
	[switch]$Upgrade
	)
	Write-Verbose "Entering Install-SASWebParts"	
	$exitCode = 0
	try
	{
		Write-Verbose "Input Parameters"
		Write-Verbose "SiteUrl = $SiteUrl"
		if ( $WebApp )	
		{Write-Verbose "WebApp = $WebApp"}
		else
		{Write-Verbose "WebApp = -AllWebApplications"}
		Write-Verbose "ServiceUrl = $ServiceUrl"
		Write-Verbose "ServerName = $ServerName"
		Write-Verbose "Port = $Port"
		Write-Verbose "SASTrust = $SASTrust"
		if ( $Password )
		{Write-Verbose "Password was specified"}
		else
		{Write-Verbose "Password not specified"}
		Write-Verbose "ProfileName = $ProfileName"
		Write-Verbose "PackageFile = $PackageFile"
		if ( $ImpersonationID )
		{Write-Verbose "ImpersonationID = $ImpersonationID"}
		if ( $Upgrade )
		{Write-Verbose "Upgrade Option selected"}
		
		#start the SharePoint AdminV4 service if not running
		$exitCode = StartSharePointAdminService
		if ( $exitCode -ne 0 )
		{ 	Throw "Unable to start SharePoint AdminV4 service. Verify the SharePoint installation on this computer, and start the service" }

		#load sharepoint snapin if not installed 
		$exitCode = AddSharePointSnapIn
		if ( $exitCode -ne 0 )
		{ 	Throw "Unable to load SharePoint SnapIn. Verify SharePoint is installed on this computer" }
		
		$WasInstalled = $False
		if ( $Upgrade )
		{			
			write-host "Begin Upgrade"
			#check if installed first, if installed
			if ((Get-SPSolution $SASSolutionID ) -ne $null )
			{
				#Disable SASCentral Feature
				Write-Verbose "Disable SASCentral webpart"
				Disable-SPFeature -Identity $SASCentralFeatureID -Url $SiteUrl -Confirm:$false
				#Uninstall SASCentral Feature
				Uninstall-SPFeature -Identity $SASCentralFeatureID  -force -Confirm:$false

				#Disable SASContentViewerFeature
				Write-Verbose "Disable SASContentWebPart"
				Disable-SPFeature -Identity $SASContentViewerFeatureID -Url $SiteUrl -Confirm:$false
				#Uninstall SASContentViewerFeature
				Uninstall-SPFeature -Identity $SASContentViewerFeatureID -force -Confirm:$false
		
				#Uninstall Solution
				if ( -not $WebApp )
				{
					Uninstall-SPSolution -Identity $SASSolutionID -AllWebApplications -Confirm:$false
				}
				else
				{
					Uninstall-SPSolution -Identity $SASSolutionID -WebApplication $WebApp -Confirm:$false
				}
				$deployed = WaitForSolutionDeployState $False 120
				Write-Verbose "Solution Deployed = $deployed"		
				if ( $deployed -eq $True )
				{ 
					$exitCode = -3; 
					Throw "Uninstall of Solution failed or is still pending";
				}
				
				Write-Verbose "remove spsolution"
				Remove-SPSolution -Identity $SASSolutionID -force -Confirm:$false	
			}
			else
			{
				$Upgrade = $false;
			}
		}
		
		#make sure it is not already installed
		try
		{
			if ((Get-SPSolution $SASSolutionID 2>$null ) -ne $null )
			{	$exitCode = -3; Throw "SAS Web Parts Solution $SASSolutionID is already installed on $SiteUrl"; }
		}
		catch
		{
		    if ( $exitCode -ne -3 )
			{
				$exitCode = -18
				Throw "Unable to Install SAS Web Parts Solution: " + $_
			}
			else 
			{
				Throw $_
			}
		}
		#verify .wsp file	
		if ((get-childitem $PackageFile 2> $null) -eq $null)
		{	$exitCode = -2; Throw "Invalid path to SAS WebParts package: $packageFile"	}
		
		# Adding solution to SharePoint"
		Write-Host "Adding $PackageFile to SharePoint..."
		Add-SPSolution -LiteralPath "$PackageFile" -Confirm:$false 
		#confirm added
		Start-Sleep 5
		if ((Get-SPSolution $SASSolutionID 2> $null) -eq $null )
		{	$exitCode = -4; Throw "Unable to add $packageFile to SharePoint site at $SiteUrl"; }
		Write-Verbose "Solution $SASSolutionID was added"
		
		#deploy Solution
		if ( -not $WebApp )
		{
			Write-Host "Deploying solution SASWebParts to all webapps ..."
			Install-SPSolution -Identity $SASSolutionID -AllWebApplications -GACDeployment        
		}
		else
		{
			Write-Host "Deploying solution $SASWebParts to WebApplication $WebApp ..."
			Install-SPSolution -Identity $SASSolutionID -WebApplication $WebApp -GACDeployment -Force       
		}
		$deployed = WaitForSolutionDeployState $True 40
		if ( $deployed -eq $false )
		{ $exitCode = -5; Throw "Unable to deploy SASWebParts Solution to $WebApp"; }
		Write-Verbose "Solution Deployed"
				
		if ( $Upgrade )
		{
			Write-Host "Upgrading SAS WebParts Features"
			$exitCode = Enable-SASWebParts -url $SiteUrl -svc $ServiceUrl -server $ServerName -P $Port -id $SASTrust -pw $Password -ImpID $ImpersonationID -ProfileName $ProfileName -Upgrade
		}
		else
		{
			Write-Host "Activating SAS WebParts Features"
			$exitCode = Enable-SASWebParts -url $SiteUrl -svc $ServiceUrl -server $ServerName -P $Port -id $SASTrust -pw $Password -ImpID $ImpersonationID -ProfileName $ProfileName
		}
	}
	catch
	{
		Write-Host $_
	}
	finally
	{
		Write-Verbose "Exiting Install-SASWebParts exit code = $exitCode"
	}
	return $exitCode
}

function Get-SASInstallStatus
{
<#
.SYNOPSIS
Verifies status of SAS Solutions and features on the specified site
.DESCRIPTION
Verifies status of SAS Solutions and features on the specified site
.EXAMPLE
Get-SASInstallStatus http://mysite.com
.PARAMETER SiteUrl
The url for the SharePoint site to verify
#>
[CmdletBinding()]
param 
(
[Parameter(Mandatory=$True)]
[Alias('url')]
[string]$SiteUrl
)
	Write-Verbose "Add SharePoint SnapIn if not installed"
	$ec = AddSharePointSnapIn
	Write-Verbose "Return from AddSharePointSnapIn = $ec"
	if ( $ec -ne 0 )
	{ Throw "Unable to load SharePoint SnapIn. Verify SharePoint is installed correctly on this server" }
	
	$gc = Start-SPAssignment
	try
	{
		Write-Verbose "Get Solution Deployed State"
		$Solution = Get-SPSolution $SASSolutionID -AssignmentCollection $gc 2> $null
		if ( $Solution -eq $null )
		{
			Write-Host "SAS WebParts 6.12 Solution ID $SASSolutionID is not installed on this SharePoint Server"
		}
		else
		{
			Write-Host "SAS WebParts 6.12 Solution ID $SASSolutionID is installed on this SharePoint Server"
			Write-Host "Deployed state = " $Solution.Deployed
		}
		#Check on the list feature
		Write-Verbose "Verify Profile List is installed and populated"
		$ListFeature = Get-SPFeature -Identity $SASListFeatureID -AssignmentCollection $gc  2> $null
		if ( $ListFeature -eq $null )
		{
			Write-Host "SAS Profile List Feature is not installed on site $SiteUrl"
		}
		else
		{
			Write-Host "SAS Profile List Feature is installed on site $SiteUrl"
			Write-Host "Status = " $ListFeature.Status
		}
		#Check on the list instance
		Write-Verbose "Connecting to Site $SiteUrl to Verify List Instance"
		$SPSite = $gc | Get-SPWeb $SiteUrl 2> $null
		if ($SPSite -eq $null)
		{
			Write-Host "SiteUrl $SiteUrl not found"
		}
		else
		{
			$spl = $SPSite.Lists["$SASListInstanceID"] 2> $null
			if ( $spl -eq $null )
			{
				Write-Host "SAS Profile List $SASListInstanceID not found on site $SiteUrl"
			}
			else
			{
				Write-Host "SAS Profile List $SASListInstanceID was found on site $SiteUrl"
				$count = ($spl.Items).Count
				Write-Host "number of SASProfiles = $count"
				Write-Host "Use Get-SASProfiles to examine contents of the list"
			}
			#check on SAS Content Type
			Write-Verbose "Checking on SAS Content Type";
			$contentType = "SASWebParts - ProfileContentType"		
			$ct = $SPSite.ContentTypes[$contentType];
			if ( $ct -eq $null )
			{
				Write-Host "SAS Content Type was not found on site $SiteUrl";
			}
			else
			{
				Write-Host "SAS Content Type is installed on site $SiteUrl";
			}
		}
		#finally check on the two webparts
		#Check on the Content Viewer webpart
		Write-Verbose "Verify SAS Content Viewer WebPart is installed"
		$cvFeature = Get-SPFeature -Identity $SASContentViewerFeatureID -AssignmentCollection $gc 2> $null
		if ( $cvFeature -eq $null )
		{
			Write-Host "SAS ContentViewer WebPart Feature is not installed on site $SiteUrl"
		}
		else
		{
			Write-Host "SAS ContentViewer WebPart Feature is installed on site $SiteUrl"
			Write-Host "Status = " $cvFeature.Status
		}
		#Check on the SAS Central webpart
		Write-Verbose "Verify SAS Central WebPart is installed"
		$scFeature = Get-SPFeature -Identity $SASCentralFeatureID -AssignmentCollection $gc  2> $null
		if ( $scFeature -eq $null )
		{
			Write-Host "SAS Central WebPart Feature is not installed on site $SiteUrl"
		}
		else
		{
			Write-Host "SAS Central WebPart Feature is installed on site $SiteUrl"
			Write-Host "Status = " $scFeature.Status
		}

		$found = 0;
		try {
			$centralwp = $SPSite.GetFile("_catalogs/wp/SASCentralWebPart.webpart");
			if ( $centralwp -ne $null )
			{ 
				if ( $centralwp.exists )
				{
					Write-Verbose "SAS Central web part found"
					$found += 1;
				}
			}
		} catch {
			if ( $_.FullyQualifiedErrorId -eq "InvokeMethodOnNull" )
			{
				Write-Host "SAS Central web part not found";
			}
			else
			{
				$_;
			}
		}
		try {
			$contentwp = $SPSite.GetFile("_catalogs/wp/SASContentWebPart.webpart");
			if ( $contentwp -ne $null )
			{
				if ( $contentwp.exists )
				{
					Write-Verbose "SAS Content Viewer web part found"
					$found += 1;
				}
			}
		} catch {
			if ( $_.FullyQualifiedErrorId -eq "InvokeMethodOnNull" )
			{
				Write-Host "SAS Content Viewer web part not found";		
			}
			else
			{
				$_;
			}
		}
		write-host "There were $found SAS web parts found in the Web Part Gallery"		
	}	
	finally
	{
		Stop-SPAssignment $gc
	}
}

$SASSolutionID = "a1b7f413-fbaf-4e75-a4e4-fae96a1515fe"
$SASListFeatureID = "47960b67-aa2e-4c7c-9549-e7057afa19c3"
$SASListInstanceID = "SASWebParts - SASMDProfilelist_b5a50a30-d92d-4df1-93dd-c5e8c33eafed"
$SASContentViewerFeatureID = "b5a50a30-d92d-4df1-93dd-c5e8c33eafed"
$SASCentralFeatureID = "ab6ac9ba-433e-49cd-8892-694b6a26d28f"

Export-ModuleMember -function *SAS* -variable SAS*